<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\ObjectMapper\Metadata;

use Symfony\Component\ObjectMapper\Attribute\Map;

/**
 * Configures a class or a property to map to.
 *
 * @internal
 *
 * @author Antoine Bluchet <soyuka@gmail.com>
 */
final class Mapping extends Map
{
    public function __construct(
        ?string $target = null,
        ?string $source = null,
        mixed $if = null,
        mixed $transform = null,
        public readonly ?string $targetClass = null,
    ) {
        parent::__construct($target, $source, $if, $transform);
    }
}
