<?php

return \deepclone_from_array([
    'classes' => '',
    'objectMeta' => 0,
    'prepared' => ['Symfony\\Component\\VarExporter\\Tests\\TestClass', 'testStaticMethod'],
    'mask' => 0,
], null, true);
