<?php

return \deepclone_from_array([
    'classes' => '',
    'objectMeta' => 0,
    'prepared' => ['Symfony\\Component\\VarExporter\\Tests\\Fixtures\\PrivateFCC', '', 1, 0, 18],
    'mask' => 1,
], null, true);
