<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\PropertyInfo\Tests\Fixtures;

class DummyWithGetterSetter
{
    private array $payments = [];

    public function payments(): array
    {
        return $this->payments;
    }

    public function hasPayments(): bool
    {
        return (bool) $this->payments;
    }
}
